==> tuain-ecosystem-lib-0.0.6.tgz: package/lib/helpers/db-pool.js <==
const fs = require('fs');
const path = require('path');
const mongodb = require('mongodb');
const tunnel = require('tunnel-ssh');

const { MongoClient } = mongodb;

const LOCAL_HOST = '127.0.0.1';
const CONFIG_PATH = path.resolve(__dirname, '../../config');

const moduleErrors = {
  QUERYEXECUTION: {
    apiDispatch: ['01', 'Servicio no soportado'],
    noDatabaseDefinition: ['01', 'No existen parámetros de base de datos'],
    noDatabasePool: ['02', 'No existen pool de base de datos'],
    noQuerySet: ['03', 'No existe conjunto de consultas'],
    noQueryDefinition: ['04', 'No existe definición para la consulta'],
    unknownError: ['05', 'Error en ejecución de consulta'],
  },
};

errMgr.addModuleSet('db-pool', moduleErrors);

async function tunnelAsync(tunnelConfig) {
  return new Promise((resolve, reject) => {
    tunnel(tunnelConfig, (err, server) => {
      if (err) {
        reject(err);
      } else {
        resolve(server);
      }
    });
  });
}

async function openMongoDBPool(dbConfig, logger, errMgr) {
  const {
    user, password, database, options, additionalOptions,
    tlsPublicKeyFile, sshTunnelConfig,
  } = dbConfig;
  const tlsUsePublicKey = (dbConfig.tlsUsePublicKey === 'true') ? true
    : ((dbConfig.tlsUsePublicKey === 'false') ? false : dbConfig.tlsUsePublicKey);
  const sshTunnel = (dbConfig.sshTunnel === 'true') ? true
    : ((dbConfig.sshTunnel === 'false') ? false : dbConfig.sshTunnel);
  for (const [name, value] of Object.entries(additionalOptions ?? [])) {
    const optionValue = (value === 'true') ? true : ((value === 'false') ? false : value);
    additionalOptions[name] = optionValue;
  }
  let optionsStr = '';
  for (const [name, value] of Object.entries(options ?? [])) {
    optionsStr += `${name}=${value}&`;
  }
  optionsStr = (optionsStr.length > 0) ? optionsStr.slice(0, -1) : '';
  let { host } = dbConfig;
  let { port } = dbConfig;
  const myAdditionalOptions = { ...additionalOptions };
  if (tlsUsePublicKey) {
    myAdditionalOptions.sslCA = fs.readFileSync(path.join(CONFIG_PATH, tlsPublicKeyFile));
  }
  if (sshTunnel) {
    const privateKeyFile = path.join(CONFIG_PATH, sshTunnelConfig.privateKeyFile);
    const privateKey = fs.readFileSync(privateKeyFile);
    const tunnelConfig = {
      host: sshTunnelConfig.host,
      port: sshTunnelConfig.port,
      dstHost: host,
      dstPort: port,
      localHost: LOCAL_HOST,
      localPort: sshTunnelConfig.localPort,
      username: sshTunnelConfig.username,
      keepAlive: true,
    };
    if (sshTunnelConfig.password) {
      tunnelConfig.password = sshTunnelConfig.password;
    } else {
      tunnelConfig.privateKey = privateKey;
    }
    await tunnelAsync(tunnelConfig);
    host = LOCAL_HOST;
    port = sshTunnelConfig.localPort;
  }
  const mongoDbUri = `mongodb://${user}:${password}@${host}:${port}/${database}?${optionsStr}`;
  console.log({ mongoDbUri, myAdditionalOptions });
  return MongoClient.connect(mongoDbUri, myAdditionalOptions);
}

async function initModule() {
  const poolPromises = [];
  const poolNames = [];
  try {
    const dbNames = Object.keys(databases).filter(dbName => ((databases[dbName]?.autoOpen ?? true)
    && databases[dbName]?.dbEngine === 'mongo'));
    dbNames.forEach((dbName) => {
      const dbConfig = databases[dbName];
      let appDatabasePool = null;
      appDatabasePool = openMongoDBPool(dbConfig);
      poolPromises.push(appDatabasePool);
      poolNames.push(dbName);
    });
    if (poolPromises && poolPromises.length > 0) {
      const dbPools = await Promise.all(poolPromises);
      for (let index = 0; index < dbPools.length; index += 1) {
        const dbPool = dbPools[index];
        const poolName = poolNames[index];
        if (dbPool) {
          appDbPoolSet[poolName] = dbPool;
          appDbPoolSet[poolName].on('close', () => { console.log('-> lost connection'); });
          // appDbPoolSet[poolName].on('reconnect', () => { console.log('-> reconnected'); });
        }
      }
    }
  } catch (err) {
    logger.error(`No fue posible abrir los pools ${err.message}/${err.stack}`);
  }
}

function getDb(poolName = null) {
  const poolNames = Object.keys(appDbPoolSet);
  const dbPoolName = (!poolName && poolNames && poolNames.length === 1) ? poolNames[0] : poolName;
  return (appDbPoolSet[dbPoolName]) ? appDbPoolSet[dbPoolName].db() : null;
}

function getCnx(poolName = null) {
  const poolNames = Object.keys(appDbPoolSet);
  const dbPoolName = (!poolName && poolNames && poolNames.length === 1) ? poolNames[0] : poolName;
  return appDbPoolSet[dbPoolName];
}

module.exports = {
  initModule,
  getDb,
  ObjectId: mongodb.ObjectId,
  getCnx,
};

==> tuain-ecosystem-lib-0.0.9.tgz: package/lib/helpers/db-pool.js <==
const fs = require('fs');
const path = require('path');
const mongodb = require('mongodb');
const tunnel = require('tunnel-ssh');

const { appDbPoolSet } = '../../config';

const { MongoClient } = mongodb;

const LOCAL_HOST = '127.0.0.1';
const CONFIG_PATH = path.resolve(__dirname, '../../config');

const moduleErrors = {
  QUERYEXECUTION: {
    apiDispatch: ['01', 'Servicio no soportado'],
    noDatabaseDefinition: ['01', 'No existen parámetros de base de datos'],
    noDatabasePool: ['02', 'No existen pool de base de datos'],
    noQuerySet: ['03', 'No existe conjunto de consultas'],
    noQueryDefinition: ['04', 'No existe definición para la consulta'],
    unknownError: ['05', 'Error en ejecución de consulta'],
  },
};

async function tunnelAsync(tunnelConfig) {
  return new Promise((resolve, reject) => {
    tunnel(tunnelConfig, (err, server) => {
      if (err) {
        reject(err);
      } else {
        resolve(server);
      }
    });
  });
}

async function openMongoDBPool(dbConfig) {
  const {
    user, password, database, options, additionalOptions,
    tlsPublicKeyFile, sshTunnelConfig,
  } = dbConfig;
  const tlsUsePublicKey = (dbConfig.tlsUsePublicKey === 'true') ? true
    : ((dbConfig.tlsUsePublicKey === 'false') ? false : dbConfig.tlsUsePublicKey);
  const sshTunnel = (dbConfig.sshTunnel === 'true') ? true
    : ((dbConfig.sshTunnel === 'false') ? false : dbConfig.sshTunnel);
  for (const [name, value] of Object.entries(additionalOptions ?? [])) {
    const optionValue = (value === 'true') ? true : ((value === 'false') ? false : value);
    additionalOptions[name] = optionValue;
  }
  let optionsStr = '';
  for (const [name, value] of Object.entries(options ?? [])) {
    optionsStr += `${name}=${value}&`;
  }
  optionsStr = (optionsStr.length > 0) ? optionsStr.slice(0, -1) : '';
  let { host } = dbConfig;
  let { port } = dbConfig;
  const myAdditionalOptions = { ...additionalOptions };
  if (tlsUsePublicKey) {
    myAdditionalOptions.sslCA = fs.readFileSync(path.join(CONFIG_PATH, tlsPublicKeyFile));
  }
  if (sshTunnel) {
    const privateKeyFile = path.join(CONFIG_PATH, sshTunnelConfig.privateKeyFile);
    const privateKey = fs.readFileSync(privateKeyFile);
    const tunnelConfig = {
      host: sshTunnelConfig.host,
      port: sshTunnelConfig.port,
      dstHost: host,
      dstPort: port,
      localHost: LOCAL_HOST,
      localPort: sshTunnelConfig.localPort,
      username: sshTunnelConfig.username,
      keepAlive: true,
    };
    if (sshTunnelConfig.password) {
      tunnelConfig.password = sshTunnelConfig.password;
    } else {
      tunnelConfig.privateKey = privateKey;
    }
    await tunnelAsync(tunnelConfig);
    host = LOCAL_HOST;
    port = sshTunnelConfig.localPort;
  }
  const mongoDbUri = `mongodb://${user}:${password}@${host}:${port}/${database}?${optionsStr}`;
  console.log({ mongoDbUri, myAdditionalOptions });
  return MongoClient.connect(mongoDbUri, myAdditionalOptions);
}

async function openPoolSet(databases, logger, errMgr) {
  const poolPromises = [];
  const poolNames = [];
  errMgr.addModuleSet('db-pool', moduleErrors);
  try {
    const dbNames = Object.keys(databases).filter(dbName => ((databases[dbName]?.autoOpen ?? true)
    && databases[dbName]?.dbEngine === 'mongo'));
    dbNames.forEach((dbName) => {
      const dbConfig = databases[dbName];
      let appDatabasePool = null;
      appDatabasePool = openMongoDBPool(dbConfig);
      poolPromises.push(appDatabasePool);
      poolNames.push(dbName);
    });
    if (poolPromises && poolPromises.length > 0) {
      const dbPools = await Promise.all(poolPromises);
      for (let index = 0; index < dbPools.length; index += 1) {
        const dbPool = dbPools[index];
        const poolName = poolNames[index];
        if (dbPool) {
          appDbPoolSet[poolName] = dbPool;
          appDbPoolSet[poolName].on('close', () => { console.log('-> lost connection'); });
          // appDbPoolSet[poolName].on('reconnect', () => { console.log('-> reconnected'); });
        }
      }
    }
  } catch (err) {
    logger.error(`No fue posible abrir los pools ${err.message}/${err.stack}`);
  }
}

function getDb(poolName = null) {
  const poolNames = Object.keys(appDbPoolSet);
  const dbPoolName = (!poolName && poolNames && poolNames.length === 1) ? poolNames[0] : poolName;
  return (appDbPoolSet[dbPoolName]) ? appDbPoolSet[dbPoolName].db() : null;
}

function getCnx(poolName = null) {
  const poolNames = Object.keys(appDbPoolSet);
  const dbPoolName = (!poolName && poolNames && poolNames.length === 1) ? poolNames[0] : poolName;
  return appDbPoolSet[dbPoolName];
}

module.exports = {
  openPoolSet,
  getDb,
  ObjectId: mongodb.ObjectId,
  getCnx,
};
